@@ -1,4 +1,4 @@
-/*  $Id: api.c 2714 2006-08-22 18:31:28Z rra $
+/*  $Id: api.c 2978 2007-02-11 03:37:08Z rra $
 **
 **  Entry points for the remctl library API.
 **
@@ -8,7 +8,7 @@
 **
 **  Written by Russ Allbery <rra@stanford.edu>
 **  Based on work by Anton Ushakov
-**  Copyright 2002, 2003, 2004, 2005, 2006
+**  Copyright 2002, 2003, 2004, 2005, 2006, 2007
 **      Board of Trustees, Leland Stanford Jr. University
 **
 **  See README for licensing terms.
@@ -16,15 +16,10 @@
 
 #include <config.h>
 #include <system.h>
+#include <portable/gssapi.h>
 
 #include <errno.h>
 
-#ifdef HAVE_GSSAPI_H
-# include <gssapi.h>
-#else
-# include <gssapi/gssapi_generic.h>
-#endif
-
 #include <client/internal.h>
 #include <client/remctl.h>
 #include <util/util.h>
