@@ -1,4 +1,4 @@
-/*  $Id: generic.c 2742 2006-09-08 04:02:06Z rra $
+/*  $Id: generic.c 2978 2007-02-11 03:37:08Z rra $
 **
 **  Server implementation of generic protocol functions.
 **
@@ -7,7 +7,7 @@
 **
 **  Written by Russ Allbery <rra@stanford.edu>
 **  Based on work by Anton Ushakov
-**  Copyright 2002, 2003, 2004, 2005, 2006
+**  Copyright 2002, 2003, 2004, 2005, 2006, 2007
 **      Board of Trustees, Leland Stanford Jr. University
 **
 **  See README for licensing terms.
@@ -15,27 +15,12 @@
 
 #include <config.h>
 #include <system.h>
-
-/* BSDI needs <netinet/in.h> before <arpa/inet.h>. */
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <sys/socket.h>
-
-#ifdef HAVE_GSSAPI_H
-# include <gssapi.h>
-#else
-# include <gssapi/gssapi_generic.h>
-#endif
+#include <portable/gssapi.h>
+#include <portable/socket.h>
 
 #include <server/internal.h>
 #include <util/util.h>
 
-/* This may already be defined by the system headers. */
-#ifndef INET_ADDRSTRLEN
-# define INET_ADDRSTRLEN 16
-#endif
-
 
 /*
 **  Create a new client struct from a file descriptor and establish a GSS-API
@@ -47,10 +32,9 @@ struct client *
 server_new_client(int fd, gss_cred_id_t creds)
 {
     struct client *client;
-    struct sockaddr_in sin;
+    struct sockaddr_storage ss;
     socklen_t socklen;
     size_t length;
-    struct hostent *host;
     char *buffer;
     gss_buffer_desc send_tok, recv_tok, name_buf;
     gss_name_t name = GSS_C_NO_NAME;
@@ -72,25 +56,33 @@ server_new_client(int fd, gss_cred_id_t creds)
     client->hostname = NULL;
     client->ipaddress = NULL;
 
-    /* Fill in hostname and IP address.  FIXME: Changes needed for IPv6. */
-    socklen = sizeof(sin);
-    if (getpeername(fd, (struct sockaddr *) &sin, &socklen) != 0) {
+    /* Fill in hostname and IP address. */
+    socklen = sizeof(ss);
+    if (getpeername(fd, (struct sockaddr *) &ss, &socklen) != 0) {
         syswarn("cannot get peer address");
         goto fail;
     }
-    length = INET_ADDRSTRLEN + 1;
+    length = INET6_ADDRSTRLEN;
     buffer = xmalloc(length);
     client->ipaddress = buffer;
-    if (inet_ntop(sin.sin_family, &sin.sin_addr, buffer, length) == NULL) {
-        syswarn("cannot translate IP address of client");
+    status = getnameinfo((struct sockaddr *) &ss, socklen, buffer, length,
+                         NULL, 0, NI_NUMERICHOST);
+    if (status != 0) {
+        syswarn("cannot translate IP address of client: %s",
+                gai_strerror(status));
         goto fail;
     }
-    host = gethostbyaddr(&sin.sin_addr, sizeof(sin.sin_addr), sin.sin_family);
-    if (host != NULL)
-        client->hostname = xstrdup(host->h_name);
+    length = NI_MAXHOST;
+    buffer = xmalloc(length);
+    status = getnameinfo((struct sockaddr *) &ss, socklen, buffer, length,
+                         NULL, 0, NI_NAMEREQD);
+    if (status == 0)
+        client->hostname = buffer;
+    else
+        free(buffer);
 
     /* Accept the initial (worthless) token. */
-    status = token_recv(client->fd, &flags, &recv_tok, 64 * 1024);
+    status = token_recv(client->fd, &flags, &recv_tok, TOKEN_MAX_LENGTH);
     if (status != TOKEN_OK) {
         warn_token("receiving initial token", status, major, minor);
         goto fail;
@@ -107,7 +99,7 @@ server_new_client(int fd, gss_cred_id_t creds)
 
     /* Now, do the real work of negotiating the context. */
     do {
-        status = token_recv(client->fd, &flags, &recv_tok, 64 * 1024);
+        status = token_recv(client->fd, &flags, &recv_tok, TOKEN_MAX_LENGTH);
         if (status != TOKEN_OK) {
             warn_token("receiving context token", status, major, minor);
             goto fail;
@@ -116,6 +108,7 @@ server_new_client(int fd, gss_cred_id_t creds)
             client->protocol = 1;
         else if (flags != (TOKEN_CONTEXT | TOKEN_PROTOCOL)) {
             warn("bad token flags %d in context token", flags);
+            gss_release_buffer(&minor, &recv_tok);
             goto fail;
         }
         debug("received context token (size=%lu)",
@@ -227,11 +220,16 @@ server_parse_command(struct client *client, const char *buffer, size_t length)
     argc = ntohl(tmp);
     p += 4;
     debug("argc is %lu", (unsigned long) argc);
-    if (argc <= 0 || argc > MAXCMDARGS) {
+    if (argc <= 0) {
         warn("invalid argc %lu in request message", (unsigned long) argc);
         server_send_error(client, ERROR_BAD_COMMAND, "Invalid command token");
         return NULL;
     }
+    if (argc > MAXCMDARGS) {
+        warn("too large argc %lu in request message", (unsigned long) argc);
+        server_send_error(client, ERROR_TOOMANY_ARGS, "Too many arguments");
+        return NULL;
+    }
     if (length - (p - buffer) < 4 * argc) {
         warn("command data too short");
         server_send_error(client, ERROR_BAD_COMMAND, "Invalid command token");
@@ -242,7 +240,7 @@ server_parse_command(struct client *client, const char *buffer, size_t length)
 
     /* Parse out the arguments and store them into a vector.  Arguments are
        packed: (<arglength><argument>)+. */
-    while (p < buffer + length - 4) {
+    while (p <= buffer + length - 4) {
         memcpy(&tmp, p, 4);
         arglen = ntohl(tmp);
         p += 4;
@@ -263,6 +261,14 @@ server_parse_command(struct client *client, const char *buffer, size_t length)
         argv->strings[argv->count][arglen] = '\0';
         argv->count++;
         p += arglen;
+        debug("arg %lu has length %lu", (unsigned long) argv->count,
+              (unsigned long) arglen);
+    }
+    if (argv->count != argc || p != buffer + length) {
+        warn("argument count differs from arguments seen");
+        server_send_error(client, ERROR_BAD_COMMAND, "Invalid command token");
+        vector_free(argv);
+        return NULL;
     }
     return argv;
 }
@@ -282,9 +288,10 @@ server_send_error(struct client *client, enum error_codes error,
     else {
         if (client->output != NULL)
             free(client->output);
-        client->output = xmalloc(strlen(message));
+        client->output = xmalloc(strlen(message) + 1);
         memcpy(client->output, message, strlen(message));
-        client->outlen = strlen(message);
+        client->output[strlen(message)] = '\n';
+        client->outlen = strlen(message) + 1;
         return server_v1_send_output(client, -1);
     }
 }
