@@ -1,12 +1,13 @@
-/* $Id: open-t.c 2725 2006-08-22 23:19:12Z rra $ */
+/* $Id: open-t.c 2978 2007-02-11 03:37:08Z rra $ */
 /* Test suite for the client connection negotiation code. */
 
 /* Written by Russ Allbery <rra@stanford.edu>
-   Copyright 2006 Board of Trustees, Leland Stanford Jr. University
+   Copyright 2006, 2007 Board of Trustees, Leland Stanford Jr. University
    See README for licensing terms. */
 
 #include <config.h>
 #include <system.h>
+#include <portable/gssapi.h>
 
 #include <fcntl.h>
 #include <netinet/in.h>
@@ -15,23 +16,6 @@
 #include <sys/time.h>
 #include <sys/wait.h>
 
-#ifdef HAVE_GSSAPI_H
-# include <gssapi.h>
-#else
-# include <gssapi/gssapi_generic.h>
-#endif
-
-/* Handle compatibility to older versions of MIT Kerberos. */
-#ifndef HAVE_GSS_RFC_OIDS
-# define GSS_C_NT_USER_NAME gss_nt_user_name
-#endif
-
-/* Heimdal provides a nice #define for this. */
-#if !HAVE_DECL_GSS_KRB5_MECHANISM
-# include <gssapi/gssapi_krb5.h>
-# define GSS_KRB5_MECHANISM gss_mech_krb5
-#endif
-
 #include <client/internal.h>
 #include <client/remctl.h>
 #include <tests/libtest.h>
