@@ -1,12 +1,13 @@
-/* $Id: tokens-t.c 2671 2006-08-08 03:00:48Z rra $ */
+/* $Id: tokens-t.c 2978 2007-02-11 03:37:08Z rra $ */
 /* tokens test suite. */
 
 /* Written by Russ Allbery <rra@stanford.edu>
-   Copyright 2006 Board of Trustees, Leland Stanford Jr. University
+   Copyright 2006, 2007 Board of Trustees, Leland Stanford Jr. University
    See README for licensing terms. */
 
 #include <config.h>
 #include <system.h>
+#include <portable/gssapi.h>
 
 #include <fcntl.h>
 #include <netinet/in.h>
@@ -15,12 +16,6 @@
 #include <sys/time.h>
 #include <sys/wait.h>
 
-#ifdef HAVE_GSSAPI_H
-# include <gssapi.h>
-#else
-# include <gssapi/gssapi_generic.h>
-#endif
-
 #include <tests/libtest.h>
 #include <util/util.h>
 
@@ -117,7 +112,7 @@ main(void)
 
     alarm(2);
 
-    test_init(9);
+    test_init(10);
 
     unlink("server-ready");
     child = fork();
@@ -197,7 +192,21 @@ main(void)
         ok_int(9, TOKEN_FAIL_EOF, status);
         waitpid(child, NULL, 0);
     }
-
     unlink("server-ready");
+
+    /* Special test for error handling when sending tokens. */
+    server = open("/dev/full", O_RDWR);
+    if (server < 0)
+        skip(10, "/dev/full not available");
+    else {
+        result.value = xmalloc(5);
+        memcpy(result.value, "hello", 5);
+        result.length = 5;
+        status = token_send(server, 3, &result);
+        free(result.value);
+        ok_int(10, TOKEN_FAIL_SYSTEM, status);
+        close(server);
+    }
+
     return 0;
 }
